using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class MouseInMap : MonoBehaviour
{
    public Camera playCamera;
    public GameObject selectEffect;
    public GameObject PlayGameModeControllerObj;
    public GameObject EnemyContainerObj;
    public GameObject SceneBlockContainerObj;
    public GameObject HUDObj;

    private Vector3 mouseInMapPosition = Vector3.zero;
    private Vector3 nowHitPosition = Vector3.zero;
    private LayerMask groundMask;
    private int randBlockNum;
    private GameObject preSet;
    private PlayGameModeController gameModeCon;
    private MousePreview mousePreviewCon;
    private EnemyContainer enemyCon;
    private SceneBlockContainer sceneBlockCon;
    private TargetUIController targetUICon;

    public enum MouseMode
    {
        Default,
        AttackSet,
        GotoSet,
        EnemySet
    }
    public MouseMode mouseMode = MouseMode.Default;

    private void Start()
    {
        groundMask = LayerMask.GetMask("Ground");
        gameModeCon = PlayGameModeControllerObj.GetComponent<PlayGameModeController>();
        mousePreviewCon = this.GetComponent<MousePreview>();
        enemyCon = EnemyContainerObj.GetComponent<EnemyContainer>();
        sceneBlockCon = SceneBlockContainerObj.GetComponent<SceneBlockContainer>();
        targetUICon = HUDObj.GetComponent<TargetUIController>();
    }

    void Update()
    {
        nowHitPosition = getMouseOnMapPosition();
        // if mouse position in area, update mouseInMapPosition as nowHitPosition
        if (nowHitPosition.x < gameModeCon.maxAgentAreaX && nowHitPosition.x > gameModeCon.minAgentAreaX && nowHitPosition.z < gameModeCon.maxEnemyAreaZ && nowHitPosition.z > gameModeCon.minAgentAreaZ)
        {
            mouseInMapPosition = nowHitPosition;
            mousePreviewCon.updatePreviewPosition(mouseInMapPosition);
            // Mouse button R pressed
            if (Input.GetMouseButtonDown(1))
            {
                switch(mouseMode)
                {
                    case MouseMode.AttackSet:
                        sceneBlockCon.createNewBlock(SceneBlockContainer.Targets.Attack, randBlockNum, mouseInMapPosition);
                        changeMouseModeTo(MouseMode.Default);
                        break;
                    case MouseMode.GotoSet:
                        sceneBlockCon.createNewBlock(SceneBlockContainer.Targets.Go, randBlockNum, mouseInMapPosition);
                        changeMouseModeTo(MouseMode.Default);
                        break;
                    case MouseMode.EnemySet:
                        enemyCon.initEnemyAtHere(new Vector3(mouseInMapPosition.x,1,mouseInMapPosition.z));
                        break;
                    default:
                        break;
                }
            }
        }
    }

    public void changeMouseModeTo(MouseMode thisMouseMode)
    {
        mouseMode = thisMouseMode;
        switch (thisMouseMode)
        {
            case MouseMode.AttackSet:
                // random choose attack scene block type and set as preview
                randBlockNum = Random.Range(0, sceneBlockCon.attackBlockPrefabs.Length);
                preSet = sceneBlockCon.goBlockPrefabs[randBlockNum];
                mousePreviewCon.changePreviewTo(preSet);
                break;
            case MouseMode.GotoSet:
                // random choose Goto scene block type and set as preview
                randBlockNum = Random.Range(0, sceneBlockCon.goBlockPrefabs.Length);
                preSet = sceneBlockCon.goBlockPrefabs[randBlockNum];
                mousePreviewCon.changePreviewTo(preSet);
                break;
            case MouseMode.EnemySet:
                preSet = enemyCon.enemyPrefab;
                mousePreviewCon.changePreviewTo(preSet);
                break;
            default:
                mousePreviewCon.deleteAllPreviewModele();
                break;
        }
    }

    // get mouse position on map, return a Vector3
    public Vector3 getMouseOnMapPosition()
    {
        // shoot raycast from mainCamera center to mousepositon
        RaycastHit thisHit;
        Color rayColor = Color.red;
        Vector3 onMapPosition;
        Ray ray = playCamera.ScreenPointToRay(Input.mousePosition);
        // if raycast hit gameobject
        if (Physics.Raycast(ray, out thisHit, Mathf.Infinity, groundMask))
        {
            //draw raycast
            Debug.DrawRay(ray.origin, ray.direction * 100, rayColor);
            return onMapPosition = thisHit.point;
        }
        else
        {
            return Vector3.zero;
        }
    }
}
